/******************************** LICENSE ********************************


 Copyright 2007 European Centre for Medium-Range Weather Forecasts (ECMWF)
 
 Licensed under the Apache License, Version 2.0 (the "License"); 
 you may not use this file except in compliance with the License. 
 You may obtain a copy of the License at 
 
 	http://www.apache.org/licenses/LICENSE-2.0
 
 Unless required by applicable law or agreed to in writing, software 
 distributed under the License is distributed on an "AS IS" BASIS, 
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 See the License for the specific language governing permissions and 
 limitations under the License.


 ******************************** LICENSE ********************************/

/*!
    \file SciMethods.h
    \brief Definition of SciMethods.
    \author Graphics Section, ECMWF

    Started: September 2011
*/

#ifndef SciMethods_H
#define SciMethods_H


#include "magics.h"

namespace magics
{
/*! Earth radius in km */
const double EarthRadiusInKm=6378.388;   //Earth radius in km 

/*! Computes the distance on Earth in km */
double geoDistanceInKm(double,double,double,double);

/*!  computes potential temperature */
double theta(double t, double p);

/*!  computes temperature form potential temperature */
double temperatureFromTheta(double th, double p);

/*!  computes pressure potential temperature */
double pressureFromTheta(double th, double t);

double tDew(double t, double rh);
double tWet(double td,double t,double p);
double thetaEq(double td,double t,double p);
double thetaEq(double t,double p);
double temperatureFromThetaEq(double thSat,double p);
double saturationVapourPressure(double t);
double mixingRatio(double t, double p);
double temperatureFromMixingRatio(double r, double p);
double temperatureLCL(double td,double t);
double pressureLCL(double td, double t, double p);
}

#endif